package dev.fyloz.trial.colorrecipesexplorer.core.services;

import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Mix;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MixQuantity;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import org.springframework.stereotype.Service;

import java.util.Map;
import java.util.stream.Collectors;

@Service
public class InventoryService {

    private MaterialService materialService;

    public InventoryService(MaterialService materialService) {
        this.materialService = materialService;
    }

    public String checkQuantities(Mix mix, Map<Material, Float> quantities) {
        for (Material material : mix.getMixQuantities().stream().map(MixQuantity::getMaterial).collect(Collectors.toList())) {
            if (!material.isMixType()) {
                Float quantity = quantities.get(material);

                if (quantity > material.getInventoryQuantity()) {
                    return String.format("%s-%s", mix.getId(), material.getId());
                }
            }
        }

        return null;
    }

    public boolean useMix(Map<Material, Float> quantities) {
        for (Map.Entry<Material, Float> entry : quantities.entrySet()) {
            Material material = entry.getKey();

            if (!material.isMixType()) {
                material.setInventoryQuantity(material.getInventoryQuantity() - entry.getValue());
                if (materialService.update(material).isEmpty()) return false;
            }
        }

        return true;
    }
}
