package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.dao.MaterialTypeDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
public class MaterialTypeService extends GenericService<MaterialType, MaterialTypeDao> {

    private MaterialService materialService;

    @Autowired
    public MaterialTypeService(MaterialTypeDao materialTypeDao, MaterialService materialService) {
        super(materialTypeDao);
        this.materialService = materialService;
    }

    public boolean isLinkedToMaterials(MaterialType materialType) {
        return !materialService.getAllByMaterialType(materialType).isEmpty();
    }

    public boolean deleteIfNotLinked(MaterialType materialType) {
        if (!isLinkedToMaterials(materialType)) {
            return delete(materialType);
        }

        return false;
    }

    @Override
    public boolean isValidForCreation(MaterialType entity) {
        return entity != null && !existsByName(entity.getName());
    }

    @Override
    public boolean isValidForUpdate(MaterialType materialType) {
        return super.isValidForUpdate(materialType)
                && isValidForUpdateName(materialType)
                && isValidForUpdatePrefix(materialType);
    }

    public boolean isValidForUpdateName(MaterialType materialType) {
        Optional<MaterialType> materialTypeByName = dao.findByName(materialType.getName());

        return materialTypeByName.isEmpty() || materialType.getId().equals(materialTypeByName.get().getId());
    }

    public boolean isValidForUpdatePrefix(MaterialType materialType) {
        Optional<MaterialType> materialTypeByPrefix = dao.findByPrefix(materialType.getPrefix());

        return materialTypeByPrefix.isEmpty() || materialType.getId().equals(materialTypeByPrefix.get().getId());
    }

    public Optional<MaterialType> getByName(String name) {
        return dao.findByName(name);
    }

    // TODO Utilisé ?
    public Optional<MaterialType> getDefaultMaterialType() {
        return getByName(MaterialType.DEFAULT_MATERIAL_TYPE.getName());
    }

    public boolean existsByName(String name) {
        return getByName(name).isPresent();
    }
}
