package dev.fyloz.trial.colorrecipesexplorer.web.controller.creators;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.CompanyService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;
import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.CREATOR_RECIPE;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.EDITOR_RECIPE_SPECIFIC;

@Controller
public class RecipeCreatorController {

    private RecipeService recipeService;
    private CompanyService companyService;

    @Autowired
    public RecipeCreatorController(RecipeService recipeService, CompanyService companyService) {
        this.recipeService = recipeService;
        this.companyService = companyService;
    }

    @GetMapping(CREATOR_RECIPE)
    public ModelAndView showCreationPage(ModelAndView model, Recipe recipe) {
        ModelResponseBuilder responseBuilder = new ModelResponseBuilder(model)
                .withView(CREATOR_RECIPE)
                .addResponseData(ResponseDataType.COMPANIES, companyService.getAll())
                .addResponseData(ResponseDataType.RECIPE, recipe == null ? new Recipe() : recipe);

        if (companyService.getAll().isEmpty()) {
            responseBuilder.addResponseCode(ResponseCode.NO_COMPANY)
                    .addResponseData(ResponseDataType.BLOCK_BUTTON, true);
        }

        return responseBuilder.build();
    }

    /**
     * Permet à l'utilisateur de créer une nouvelle recette.
     * <p>
     * La création échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Une erreur est survenue lors de la sauvegarde dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param recipe La recette à créer
     * @return La page à afficher.
     */
    @PostMapping(value = CREATOR_RECIPE)
    public ModelAndView createRecipe(@Valid Recipe recipe) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder();

        Optional<Recipe> savedRecipe = recipeService.save(recipe);
        if (savedRecipe.isPresent()) {
            return modelResponseBuilder
                    .withRedirect(EDITOR_RECIPE_SPECIFIC, savedRecipe.get().getRecipeID())
                    .build();
        }
        modelResponseBuilder.addResponseCode(ResponseCode.ERROR_SAVING);

        return showCreationPage(modelResponseBuilder.build(), recipe);
    }
}
