package dev.fyloz.trial.colorrecipesexplorer.web.controller.editors;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.*;

@Controller
public class MaterialTypeEditorController {

    private MaterialTypeService materialTypeService;

    @Autowired
    public MaterialTypeEditorController(MaterialTypeService materialTypeService) {
        this.materialTypeService = materialTypeService;
    }

    @GetMapping(EDITOR_MATERIAL_TYPE)
    public ModelAndView listMaterialTypes(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(EDITOR_MATERIAL_TYPE)
                .addResponseData(ResponseDataType.MATERIAL_TYPES, materialTypeService.getAll())
                .build();
    }

    @GetMapping(EDITOR_MATERIAL_TYPE_SPECIFIC)
    public ModelAndView showEditPage(ModelAndView model, @PathVariable Long id, MaterialType materialType) {
        ModelResponseBuilder responseBuilder = new ModelResponseBuilder(model)
                .withView(EDITOR_MATERIAL_TYPE_EDITOR);

        if (materialType.getName() == null) {
            Optional<MaterialType> optionalMaterialType = materialTypeService.getById(id);

            if (optionalMaterialType.isEmpty()) {
                return listMaterialTypes(
                        responseBuilder
                                .addResponseCode(ResponseCode.MATERIAL_TYPE_NOT_FOUND, id)
                                .build()
                );
            }

            materialType = optionalMaterialType.get();
        }

        return responseBuilder
                .addResponseData(ResponseDataType.MATERIAL_TYPE, materialType)
                .build();
    }

    @PostMapping(value = EDITOR_MATERIAL_TYPE, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView saveEditedMaterialType(MaterialType materialType) {
        ModelResponseBuilder responseBuilder = new ModelResponseBuilder("");

        // L'ID est 0 lors de la désérialisation, il faut utiliser le nom unique
        String materialTypeName = materialType.getName();
        Optional<MaterialType> optionalMaterialType = materialTypeService.getByName(materialTypeName);

        if (optionalMaterialType.isEmpty()) {
            responseBuilder.addResponseCode(ResponseCode.MATERIAL_TYPE_NOT_FOUND, materialTypeName);
        } else {
            materialType = optionalMaterialType.get();
            if (!materialTypeService.isValidForUpdateName(materialType)) {
                return showEditPage(
                        responseBuilder
                                .addResponseCode(ResponseCode.MATERIAL_TYPE_ALREADY_EXIST, materialType.getName())
                                .build(),
                        materialType.getMaterialTypeID(), materialType);
            } else if (!materialTypeService.isValidForUpdatePrefix(materialType)) {
                return showEditPage(
                        responseBuilder
                                .addResponseCode(ResponseCode.MATERIAL_TYPE_ALREADY_EXIST_PREFIX, materialType.getPrefix())
                                .build(),
                        materialType.getId(), materialType);
            } else {
                Optional<MaterialType> updatedMaterialType = materialTypeService.update(materialType);
                if (updatedMaterialType.isPresent()) {
                    responseBuilder.addResponseCode(ResponseCode.SUCCESS_SAVING_MATERIAL_TYPE, updatedMaterialType.get().getName());
                } else {
                    return showEditPage(
                            responseBuilder
                                    .addResponseCode(ResponseCode.ERROR_SAVING)
                                    .build(),
                            materialType.getId(), materialType);
                }
            }
        }

        return listMaterialTypes(responseBuilder.build());
    }
}
