package dev.fyloz.trial.colorrecipesexplorer.web.controller.files;

import dev.fyloz.trial.colorrecipesexplorer.ColorRecipesExplorerApplication;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import dev.fyloz.trial.colorrecipesexplorer.xlsx.XlsxExporter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.Optional;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.ALL_RECIPES_XLS;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.RECIPE_XLS;

// TODO Une grande partie du code de ce controlleur devrait se trouver dans un service
@Controller
public class XlsExporterController {

    private RecipeService recipeService;

    @Autowired
    public XlsExporterController(RecipeService recipeService) {
        this.recipeService = recipeService;
    }

    @GetMapping(RECIPE_XLS)
    public ResponseEntity<byte[]> getXlsForRecipe(HttpServletRequest request, @PathVariable int recipeID) {
        HttpHeaders headers = new HttpHeaders();

        Optional<Recipe> optionalRecipe = recipeService.getByID(recipeID);
        if (optionalRecipe.isEmpty()) {
            headers.add(HttpHeaders.LOCATION, request.getHeader("referer"));
            return new ResponseEntity<>(headers, HttpStatus.FOUND);
        }

        byte[] recipeXLS = new XlsxExporter().generate(optionalRecipe.get());
        if (recipeXLS.length <= 0) return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();

        return ResponseEntity.ok()
                .headers(headers)
                .contentLength(recipeXLS.length)
                .contentType(MediaType.parseMediaType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"))
                .body(recipeXLS);
    }

    @GetMapping(value = ALL_RECIPES_XLS, produces = "application/zip")
    public ResponseEntity<byte[]> getAllXls() throws IOException {
        HttpHeaders headers = new HttpHeaders();

        ColorRecipesExplorerApplication.LOGGER.info("Exportation de toutes les couleurs en XLS");

        ByteArrayOutputStream byteOS = new ByteArrayOutputStream();
        ZipOutputStream out = new ZipOutputStream(byteOS);

        Collection<Recipe> recipes = recipeService.getAll();
        for (Recipe recipe : recipes) {
            byte[] recipeXLS = new XlsxExporter().generate(recipe);
            if (recipeXLS.length <= 0) return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();

            out.putNextEntry(new ZipEntry(String.format("%s_%s.xlsx", recipe.getCompany().getCompanyName(), recipe.getRecipeCode())));
            out.write(recipeXLS, 0, recipeXLS.length);
            out.closeEntry();
        }

        out.close();

        byte[] zipContent = byteOS.toByteArray();
        byteOS.close();

        return ResponseEntity.ok()
                .headers(headers)
                .contentLength(zipContent.length)
                .contentType(MediaType.parseMediaType("application/zip"))
                .body(zipContent);
    }

}
