package dev.fyloz.trial.colorrecipesexplorer.web.controller.removers;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.REMOVER_MATERIAL_TYPE;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.REMOVER_MATERIAL_TYPE_SPECIFIC;

@Controller
public class MaterialTypeRemoverController {

    private MaterialTypeService materialTypeService;

    @Autowired

    public MaterialTypeRemoverController(MaterialTypeService materialTypeService) {
        this.materialTypeService = materialTypeService;
    }

    @GetMapping(REMOVER_MATERIAL_TYPE)
    public ModelAndView getPage(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(REMOVER_MATERIAL_TYPE)
                .addResponseData(ResponseDataType.MATERIAL_TYPES, materialTypeService.getAll())
                .build();
    }

    @PostMapping(REMOVER_MATERIAL_TYPE_SPECIFIC)
    public ModelAndView removeMaterialType(@PathVariable Long id) {
        ModelResponseBuilder responseBuilder = new ModelResponseBuilder("");

        Optional<MaterialType> optionalMaterialType = materialTypeService.getById(id);
        if (optionalMaterialType.isEmpty()) {
            responseBuilder.addResponseCode(ResponseCode.MATERIAL_TYPE_NOT_FOUND, id);
        } else {
            MaterialType materialType = optionalMaterialType.get();

            if (materialTypeService.deleteIfNotLinked(materialType)) {
                responseBuilder.addResponseCode(ResponseCode.SUCCESS_DELETING_MATERIAL_TYPE, materialType.getName());
            } else {
                responseBuilder.addResponseCode(ResponseCode.MATERIAL_TYPE_LINKED, materialType.getName());
            }
        }

        return getPage(responseBuilder.build());
    }
}
